#include <windows.h>
#include <windowsx.h>
#include "tabswitcher.h"

void DrawGradient(HDC dc, const RECT& rectClient, COLORREF m_clrStart, COLORREF m_clrEnd) 
{ 
	int nMaxWidth = rectClient.bottom - rectClient.top;
	RECT rectFill; // Rectangle for filling band 
	float fStep; // How wide is each band? 
	HBRUSH brush = NULL; // Brush to fill in the bar 
	//CMemDC memDC(pDC); 
	//First find out the largest color distance between the start and 
	//end colors.This distance will determine how many steps we use to 
	//carve up the client region and the size of each gradient rect. 

	int r, g, b; // First distance, then starting value 
	float rStep, gStep, bStep; // Step size for each color 
	BOOL bSameColor = FALSE; 
	int nSteps; 
	// Get the color differences 
	r = (GetRValue(m_clrEnd) - GetRValue(m_clrStart)); 
	g = (GetGValue(m_clrEnd) - GetGValue(m_clrStart)); 
	b = (GetBValue(m_clrEnd) - GetBValue(m_clrStart)); 

	if((r == 0) && (g == 0) && (b == 0)) 
	{ 
		bSameColor = TRUE; 
		//{{ADDED BY RAJESH 
		//Added the three lines below to fix the drawing 
		//problem which used to occur when both the start 
		//and end colors are same. 
		r = GetRValue(m_clrStart); 
		g = GetGValue(m_clrStart); 
		b = GetBValue(m_clrStart); 
		//}}ADDED BY RAJESH 
	} 

	// Make the number of steps equal to the greatest distance 
	//{{MODIFIED BY RAJESH 
	//To fix the drawing problem which used to occur when 
	//both the start and end colors are RGB(0,0,0). 
	if(bSameColor && m_clrStart == 0) 
	{
		nSteps = 255;//Select max. possible value for nSteps 
	}
	else 
	{
		nSteps = max(abs(r), max(abs(g), abs(b))); 
	}
	//Determine how large each band should be in order to cover the 
	//client with nSteps bands (one for every color intensity level) 
	//}}MODIFIED BY RAJESH 
	fStep = (float)(rectClient.bottom - rectClient.top) / (float)nSteps; 

	//Calculate the step size for each color 
	rStep = r/(float)nSteps; 
	gStep = g/(float)nSteps; 
	bStep = b/(float)nSteps; 

	//Reset the colors to the starting position 
	r = GetRValue(m_clrStart); 
	g = GetGValue(m_clrStart); 
	b = GetBValue(m_clrStart); 

	// Start filling bands 
	for (int iOnBand = 0; iOnBand < nSteps; iOnBand++) 
	{ 
		SetRect(&rectFill, 
			rectClient.left, 
			rectClient.top + (int)(iOnBand * fStep), 
			rectClient.right, 
			rectClient.top + (int)((iOnBand+1) * fStep));

		if(iOnBand+1 == nSteps)
		{
			rectFill.bottom = rectClient.bottom;
		}

		if(rectFill.bottom > rectClient.bottom) 
		{ 
			rectFill.bottom = rectClient.bottom;
		}

		//CDC::FillSolidRect is faster, but it does not handle 
		//8-bit color depth 
		if(bSameColor) 
		{
			brush = CreateSolidBrush(m_clrStart);
		}
		else 
		{
			brush = CreateSolidBrush(RGB(r+rStep * iOnBand, 
				g + gStep * iOnBand, b + bStep * iOnBand)); 
		}
		FillRect(dc, &rectFill, brush); 
		DeleteObject(brush);

		//If we are past the maximum for the current position we 
		//need to get out of the loop.Before we leave,we repaint the 
		//remainder of the client area with the background color. 
	} 
} 


void DrawGradientHorz(HDC dc, const RECT& rectClient, COLORREF m_clrStart, COLORREF m_clrEnd) 
{ 
	int nMaxWidth = rectClient.right - rectClient.left;
	RECT rectFill; // Rectangle for filling band 
	float fStep; // How wide is each band? 
	HBRUSH brush = NULL; // Brush to fill in the bar 
	//CMemDC memDC(pDC); 
	//First find out the largest color distance between the start and 
	//end colors.This distance will determine how many steps we use to 
	//carve up the client region and the size of each gradient rect. 

	int r, g, b; // First distance, then starting value 
	float rStep, gStep, bStep; // Step size for each color 
	BOOL bSameColor = FALSE; 
	int nSteps; 
	// Get the color differences 
	r = (GetRValue(m_clrEnd) - GetRValue(m_clrStart)); 
	g = (GetGValue(m_clrEnd) - GetGValue(m_clrStart)); 
	b = (GetBValue(m_clrEnd) - GetBValue(m_clrStart)); 

	if((r == 0) && (g == 0) && (b == 0)) 
	{ 
		bSameColor = TRUE; 
		//{{ADDED BY RAJESH 
		//Added the three lines below to fix the drawing 
		//problem which used to occur when both the start 
		//and end colors are same. 
		r = GetRValue(m_clrStart); 
		g = GetGValue(m_clrStart); 
		b = GetBValue(m_clrStart); 
		//}}ADDED BY RAJESH 
	} 

	// Make the number of steps equal to the greatest distance 
	//{{MODIFIED BY RAJESH 
	//To fix the drawing problem which used to occur when 
	//both the start and end colors are RGB(0,0,0). 
	if(bSameColor && m_clrStart == 0) 
	{
		nSteps = 255;//Select max. possible value for nSteps 
	}
	else 
	{
		nSteps = max(abs(r), max(abs(g), abs(b))); 
	}
	//Determine how large each band should be in order to cover the 
	//client with nSteps bands (one for every color intensity level) 
	//}}MODIFIED BY RAJESH 
	fStep = (float)(rectClient.right - rectClient.left) / (float)nSteps; 

	//Calculate the step size for each color 
	rStep = r/(float)nSteps; 
	gStep = g/(float)nSteps; 
	bStep = b/(float)nSteps; 

	//Reset the colors to the starting position 
	r = GetRValue(m_clrStart); 
	g = GetGValue(m_clrStart); 
	b = GetBValue(m_clrStart); 

	// Start filling bands 
	for (int iOnBand = 0; iOnBand < nSteps; iOnBand++) 
	{ 
		SetRect(&rectFill, 
			rectClient.left + (int)(iOnBand * fStep), 
			rectClient.top, 
			rectClient.left + (int)((iOnBand+1) * fStep), 
			rectClient.bottom);

		if(rectFill.right > rectClient.right) 
		{ 
			rectFill.right = rectClient.right;
		}

		//CDC::FillSolidRect is faster, but it does not handle 
		//8-bit color depth 
		if(bSameColor) 
		{
			brush = CreateSolidBrush(m_clrStart);
		}
		else 
		{
			brush = CreateSolidBrush(RGB(r+rStep * iOnBand, 
				g + gStep * iOnBand, b + bStep * iOnBand)); 
		}
		FillRect(dc, &rectFill, brush); 
		DeleteObject(brush);
	} 
} 


VOID CALLBACK TrackMouseTimerProc(HWND hWnd,UINT uMsg,UINT idEvent,DWORD dwTime) 
{
	RECT rect;
	POINT pt;

	GetWindowRect(hWnd, &rect);
	GetCursorPos(&pt);

	BOOL isMenu = FALSE;
	{
		TCHAR clsInfo[255];
		clsInfo[0] = 0;
		HWND ptWND = WindowFromPoint(pt);
		GetClassName(ptWND, clsInfo, 255);
		if(!lstrcmp(clsInfo, TEXT("#32768")))
		{
			isMenu = TRUE;
		}
		if(!lstrcmp(clsInfo, TEXT("CPPTooltip")))
		{
			isMenu = TRUE;
		}
	}

	if (!PtInRect(&rect, pt) || (WindowFromPoint(pt) != hWnd && !isMenu)) 
	{
		KillTimer(hWnd, idEvent);
		PostMessage(hWnd, WM_MOUSELEAVE, 0, 0);
	}
}


void TrackMouseLeave(HWND hWnd, int remove)
{
	if(remove)
	{
		KillTimer(hWnd, 100);
	} else
	{
		SetTimer(hWnd, 100, 50,(TIMERPROC)TrackMouseTimerProc);
	}
}


void RegisterTabSwitcherClass(HINSTANCE hInst)
{
	WNDCLASS wc;
	if(!GetClassInfo(hInst, TABSWITCHER_CLASS, &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc    = (WNDPROC)CTabSwitcher::WndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground  = NULL;//(HBRUSH)(COLOR_WINDOW + 1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = TABSWITCHER_CLASS;

		RegisterClass(&wc);
	}
	if(!GetClassInfo(hInst, TABLABEL_CLASS, &wc))
	{
		ZeroMemory(&wc, sizeof(wc));
		wc.style          = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
		wc.lpfnWndProc    = (WNDPROC)CTabLabel::WndProc;
		wc.cbClsExtra     = 0;
		wc.cbWndExtra     = 0;
		wc.hInstance      = hInst;
		wc.hIcon          = NULL;
		wc.hCursor        = LoadCursor(NULL, IDC_ARROW);
		wc.hbrBackground  = (HBRUSH)(COLOR_HIGHLIGHT + 1);
		wc.lpszMenuName   = NULL;
		wc.lpszClassName  = TABLABEL_CLASS;

		RegisterClass(&wc);
	}
}


CTabSwitcher::CTabSwitcher(void)
{
	m_BtnSize = 48;
	m_hWnd = NULL;
	m_hFont = NULL;
	m_mouseTrackInstalled = FALSE;
}

CTabSwitcher::~CTabSwitcher(void)
{
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(m_items[i].label) delete m_items[i].label;
		if(m_items[i].img) delete m_items[i].img;
	}
	m_items.Clear();
	DeleteObject(m_hFont);
}

LRESULT CALLBACK CTabSwitcher::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CTabSwitcher* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CTabSwitcher*)GetProp(hWnd, TEXT("CTabSwitcher_this"));
	}

	if(pThis || uMessage == WM_CREATE)
	{
		switch (uMessage)
		{
		case WM_MOUSELEAVE:
			pThis->OnMouseLeave();
			return 0;
		case WM_NCPAINT:
			{
				RECT rc;
				GetWindowRect(hWnd, &rc);
				rc.bottom = rc.bottom - rc.top;
				rc.right = rc.right - rc.left;
				rc.top = rc.left = 0;

				HDC hdc = GetWindowDC(hWnd);

				if(!hdc)
				{
					return DefWindowProc(hWnd, uMessage, wParam, lParam);
				}
				
				RECT rcDraw = rc;
				rcDraw.bottom = rcDraw.top + 1;
				FillRect(hdc, &rcDraw, (HBRUSH) (COLOR_GRAYTEXT + 1));

				rcDraw = rc;
				rcDraw.top = rcDraw.bottom - 1;
				FillRect(hdc, &rcDraw, (HBRUSH) (COLOR_GRAYTEXT + 1));

				rcDraw = rc;
				rcDraw.right = rcDraw.left + 1;
				FillRect(hdc, &rcDraw, (HBRUSH) (COLOR_GRAYTEXT + 1));

				rcDraw = rc;
				rcDraw.left = rcDraw.right - 1;
				FillRect(hdc, &rcDraw, (HBRUSH) (COLOR_GRAYTEXT + 1));

				ReleaseDC(hWnd, hdc);
			}
			return 0;
		case WM_NCCALCSIZE:
			return pThis->OnNCCalcSize(wParam, lParam);
		case TSWM_GETSELECTED:
			{
				for(int i=0; i < pThis->m_items.GetCount(); i++)
				{
					if(pThis->m_items[i].state & TSW_STATE_SELECTED)
					{
						return pThis->m_items[i].lParam;
					}
				}
			}
			return -1;
		case WM_LBUTTONDOWN:
			pThis->OnLButtonDown(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return 0;
		case WM_MOUSEMOVE:
			pThis->OnMouseMove(GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam));
			return 0;
		case TSWM_INSERTITEM:
			pThis->OnInsertItem((TSWITEM*) lParam);
			return 0;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = new CTabSwitcher;
				SetProp(hWnd, TEXT("CTabSwitcher_this"), (HANDLE) pThis);
				pThis->m_hWnd = hWnd;
				pThis->OnCreate();
			}
			break;
		case WM_DESTROY:
			delete pThis;
			RemoveProp(hWnd, TEXT("CTabSwitcher_this"));
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(pThis->m_hWnd, &ps);
				HFONT oldfnt = (HFONT) SelectObject(ps.hdc, pThis->m_hFont);
				pThis->OnPaint(hdc);
				SelectObject(ps.hdc, oldfnt);
				EndPaint(pThis->m_hWnd, &ps);
			}
			return 0;
		}
	}

	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CTabSwitcher::OnPaint(HDC hdc)
{
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	int y = rcClient.top;
	for(int i=0; i < m_items.GetCount(); i++)
	{
		HDC memDC = CreateCompatibleDC(hdc);
		HBITMAP bmp = CreateCompatibleBitmap(hdc, rcClient.right - rcClient.left, m_items[i].rcItem.bottom - y);
		HBITMAP oldBitmap = (HBITMAP) SelectObject(memDC, bmp);
		HFONT oldFont = (HFONT) SelectObject(memDC, m_hFont);
		RECT rcButton = {0, 0, 0, 0};
		rcButton.right = rcClient.right - rcClient.left;
		rcButton.bottom = m_items[i].rcItem.bottom - y;
		FillRect(memDC, &rcButton, (HBRUSH) (COLOR_WINDOW + 1));

		RECT rcItem = {0, 0, 0, 0};
		rcItem.left = rcClient.left + m_items[i].rcItem.left;
		rcItem.top = m_items[i].rcItem.top - (y - rcClient.top);
		rcItem.right = rcItem.left + (m_items[i].rcItem.right - m_items[i].rcItem.left);
		rcItem.bottom = rcItem.top + (m_items[i].rcItem.bottom - m_items[i].rcItem.top);

		DrawItem(memDC, &m_items[i], &rcItem);

		BitBlt(hdc, rcClient.left, y,
			rcClient.right - rcClient.left,
			m_items[i].rcItem.bottom - y,
			memDC, 0, 0, SRCCOPY);
		SelectObject(memDC, oldBitmap);
		SelectObject(memDC, oldFont);
		DeleteDC(memDC);
		DeleteObject(bmp);

		y = m_items[i].rcItem.bottom;
	}
	RECT rcFill = rcClient;
	rcFill.top = y;
	FillRect(hdc, &rcFill, (HBRUSH) (COLOR_WINDOW + 1));
}

void CTabSwitcher::OnCreate(void)
{
	NONCLIENTMETRICS ncm;
 	ZeroMemory(&ncm, sizeof(ncm));
 	ncm.cbSize = sizeof(ncm);
 	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
 	m_hFont = CreateFontIndirect(&ncm.lfMenuFont);
	HDC dc = GetDC(m_hWnd);
	HFONT oldFont = (HFONT) SelectObject(dc, m_hFont);
	SIZE sz;
	GetTextExtentPoint32(dc, TEXT("QWEqtdlg"), lstrlen(TEXT("QWEqtdlg")), &sz);
	SelectObject(dc, oldFont);
    ReleaseDC(m_hWnd, dc);
	m_TextSize = sz.cy + 6;
}

void CTabSwitcher::OnInsertItem(TSWITEM* item)
{
	_TSWITEM pItem;
	pItem.img = new CxImage;
	pItem.img->LoadResource(FindResource(item->hInst, MAKEINTRESOURCE(item->imgID), TEXT("PNG")), CXIMAGE_FORMAT_PNG, item->hInst);
	pItem.label = new TCHAR[lstrlen(item->label) + 1];
	lstrcpy(pItem.label, item->label);
	pItem.lParam = item->lParam;
	pItem.state = item->state;
	m_items.Add(&pItem);
	RecalcLayout();
	InvalidateRect(m_hWnd, NULL, FALSE);
}

void CTabSwitcher::DrawItem(HDC hdc, _TSWITEM* item, LPRECT rc)
{
	FillRect(hdc, rc, (HBRUSH) (COLOR_WINDOW + 1));
	if(item->state & TSW_STATE_SELECTED)
	{
		COLORREF bgColor = GetSysColor(COLOR_HIGHLIGHT);
		COLORREF bdrColor = tswLightColor(bgColor, -50);
		HBRUSH bgBrush = CreateSolidBrush(bgColor);
		HPEN   bdrPen = CreatePen(PS_SOLID, 1, bdrColor);

		HBRUSH oldBrush = (HBRUSH) SelectObject(hdc, bgBrush);
		HPEN   oldPen	= (HPEN) SelectObject(hdc, bdrPen);
		RoundRect(hdc, rc->left, rc->top, rc->right, rc->bottom, 2, 2);
		SelectObject(hdc, oldBrush);
		SelectPen(hdc, oldPen);
		DeleteObject(bgBrush);
		DeleteObject(bdrPen);
	} else if(item->state & TSW_STATE_HOVER)
	{
		COLORREF bgColor = tswLightColor(GetSysColor(COLOR_HIGHLIGHT), 50);
		COLORREF bdrColor = tswLightColor(bgColor, -50);
		HBRUSH bgBrush = CreateSolidBrush(bgColor);
		HPEN   bdrPen = CreatePen(PS_SOLID, 1, bdrColor);

		HBRUSH oldBrush = (HBRUSH) SelectObject(hdc, bgBrush);
		HPEN   oldPen	= (HPEN) SelectObject(hdc, bdrPen);
		RoundRect(hdc, rc->left, rc->top, rc->right, rc->bottom, 2, 2);
		SelectObject(hdc, oldBrush);
		SelectPen(hdc, oldPen);
		DeleteObject(bgBrush);
		DeleteObject(bdrPen);
	}
	if(item->img)
	{
		int x = rc->left + (rc->right - rc->left) / 2 - item->img->GetWidth() / 2;
		int y = rc->top + (rc->bottom - rc->top - m_TextSize) / 2 - item->img->GetHeight() / 2;
		if(item->state & TSW_STATE_SELECTED)
		{
			x++; y++;
		}
		item->img->Draw(hdc, x, y);
	}
	RECT rcText = *rc;
	rcText.top = rcText.bottom - m_TextSize;
	if(item->state & TSW_STATE_SELECTED)
	{
		SetTextColor(hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
	} else
	{
		SetTextColor(hdc, GetSysColor(COLOR_WINDOWTEXT));
	}
	SetBkMode(hdc, TRANSPARENT);
	DrawText(hdc, item->label, -1, &rcText, DT_CENTER | DT_VCENTER | DT_SINGLELINE | DT_END_ELLIPSIS);
}

void CTabSwitcher::RecalcLayout(void)
{
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	int y = rcClient.top + 4;
	for(int i=0; i < m_items.GetCount(); i++)
	{
		m_items[i].rcItem.left = rcClient.left + 4;
		m_items[i].rcItem.right = rcClient.right - 4;
		m_items[i].rcItem.top = y;
		m_items[i].rcItem.bottom = y + m_BtnSize + 8 + m_TextSize;
		y = m_items[i].rcItem.bottom + 2;
	}
}

void CTabSwitcher::OnMouseMove(int x, int y)
{
	if(!m_mouseTrackInstalled)
	{
		m_mouseTrackInstalled = TRUE;
		TrackMouseLeave(m_hWnd, FALSE);
	}
	POINT pt;
	pt.x = x; pt.y = y;
	int hoverIDX = -1;
	BOOL rePaint = FALSE;
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(PtInRect(&m_items[i].rcItem, pt))
		{
			hoverIDX = i;
			if(!(m_items[i].state & TSW_STATE_SELECTED) && !(m_items[i].state & TSW_STATE_HOVER))
			{
				m_items[i].state |= TSW_STATE_HOVER;
				rePaint = TRUE;
			}
			break;
		}
	}
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(i != hoverIDX)
		{
			if(m_items[i].state & TSW_STATE_HOVER)
			{
				m_items[i].state &= ~TSW_STATE_HOVER;
				rePaint = TRUE;
			}
		}
	}
	if(rePaint)
	{
		InvalidateRect(m_hWnd, NULL, FALSE);
	}
}

void CTabSwitcher::OnLButtonDown(int x, int y)
{
	POINT pt;
	pt.x = x; pt.y = y;
	int hoverIDX = -1;
	BOOL rePaint = FALSE;
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(PtInRect(&m_items[i].rcItem, pt))
		{
			hoverIDX = i;
			if(!(m_items[i].state & TSW_STATE_SELECTED))
			{
				m_items[i].state |= TSW_STATE_SELECTED;
				rePaint = TRUE;
			}
			break;
		}
	}
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(i != hoverIDX)
		{
			if(m_items[i].state & TSW_STATE_SELECTED)
			{
				m_items[i].state &= ~TSW_STATE_SELECTED;
				rePaint = TRUE;
			}
		}
	}
	if(rePaint)
	{
		InvalidateRect(m_hWnd, NULL, FALSE);
	}
	if(hoverIDX >= 0)
	{
		SendMessage(GetParent(m_hWnd), WM_COMMAND, MAKEWPARAM(GetDlgCtrlID(m_hWnd), TSWN_CHANGED), (LPARAM) m_hWnd);
	}
}


LRESULT CTabSwitcher::OnNCCalcSize(WPARAM wParam, LPARAM lParam)
{
	LPRECT rc = (LPRECT) lParam;
	rc->left += 1;
	rc->right -= 1;
	rc->top += 1;
	rc->bottom -= 1;
	return 0;
}

void CTabSwitcher::OnMouseLeave(void)
{
	if(m_mouseTrackInstalled)
	{
		m_mouseTrackInstalled = FALSE;
		TrackMouseLeave(m_hWnd, TRUE);
	}
	BOOL rePaint = FALSE;
	for(int i=0; i < m_items.GetCount(); i++)
	{
		if(m_items[i].state & TSW_STATE_HOVER)
		{
			m_items[i].state &= ~TSW_STATE_HOVER;
			rePaint = TRUE;
		}
	}
	if(rePaint)
	{
		InvalidateRect(m_hWnd, NULL, FALSE);
	}
}


CTabLabel::CTabLabel()
{
}

CTabLabel::~CTabLabel()
{
	if(m_hFont) DeleteObject(m_hFont);
}

LRESULT CALLBACK CTabLabel::WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam)
{
	CTabLabel* pThis = NULL;
	if(IsWindow(hWnd))
	{
		pThis = (CTabLabel*)GetProp(hWnd, TEXT("CTabLabel_this"));
	}

	if(pThis || uMessage == WM_CREATE)
	{
		switch (uMessage)
		{
		case WM_SETTEXT:
			{
				LRESULT res = DefWindowProc(hWnd, uMessage, wParam, lParam);
				InvalidateRect(pThis->m_hWnd, NULL, TRUE);
				return res;
			}
			break;
		case WM_CREATE:
			{
				LPCREATESTRUCT lpcs = (LPCREATESTRUCT)lParam;
				pThis = new CTabLabel;
				SetProp(hWnd, TEXT("CTabLabel_this"), (HANDLE) pThis);
				pThis->m_hWnd = hWnd;
			}
			break;
		case WM_DESTROY:
			delete pThis;
			RemoveProp(hWnd, TEXT("CTabLabel_this"));
			break;
		case WM_PAINT:
			{
				PAINTSTRUCT ps;
				HDC hdc = BeginPaint(pThis->m_hWnd, &ps);
				HFONT oldfnt = (HFONT) SelectObject(ps.hdc, pThis->m_hFont);
				pThis->OnPaint(hdc);
				SelectObject(ps.hdc, oldfnt);
				EndPaint(pThis->m_hWnd, &ps);
			}
			return 0;
		}
	}

	return DefWindowProc(hWnd, uMessage, wParam, lParam);
}

void CTabLabel::OnPaint(HDC hdc)
{
	TCHAR	m_text[500];
	GetWindowText(m_hWnd, m_text, 500);
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	DrawGradient(hdc, rcClient, tswLightColor(GetSysColor(COLOR_HIGHLIGHT), 20), tswLightColor(GetSysColor(COLOR_HIGHLIGHT), -20));
	rcClient.left += 4;
	SetTextColor(hdc, GetSysColor(COLOR_HIGHLIGHTTEXT));
	SetBkColor(hdc, GetSysColor(COLOR_HIGHLIGHT));
	SetBkMode(hdc, TRANSPARENT);
	DrawText(hdc, m_text, -1, &rcClient, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
}

void CTabLabel::OnCreate()
{
	RECT rcClient;
	GetClientRect(m_hWnd, &rcClient);
	NONCLIENTMETRICS ncm;
	ZeroMemory(&ncm, sizeof(ncm));
	ncm.cbSize = sizeof(ncm);
	SystemParametersInfo(SPI_GETNONCLIENTMETRICS, ncm.cbSize, &ncm, 0);
	ncm.lfMessageFont.lfHeight = rcClient.bottom - rcClient.top;
	ncm.lfMessageFont.lfWidth = 0;
	ncm.lfMessageFont.lfWeight = FW_BOLD;
	m_hFont = CreateFontIndirect(&ncm.lfMessageFont);
}
