#pragma once
#include "arrays.h"
#include <tlbpdklib.h>

#define TABSWITCHER_CLASS	TEXT("tlbTABSWITCHER_CLASS")
#define TABLABEL_CLASS		TEXT("TLB_TABLABEL_CLASS")

#define TSW_STATE_ENABLED	0x00
#define TSW_STATE_SELECTED	0x01
#define TSW_STATE_DISABLED	0x02
#define TSW_STATE_HOVER		0x04

struct TSWITEM
{
	LPTSTR		label;
	UINT		imgID;
	HINSTANCE	hInst;
	UINT		lParam;
	UINT		state;
};

struct _TSWITEM
{
	LPTSTR		label;
	CxImage*	img;
	UINT		lParam;
	UINT		state;
	RECT		rcItem;
};

#define TSWM_INSERTITEM		(WM_USER + 1)
#define TSWM_GETSELECTED	(WM_USER + 2)


#define TSWN_CHANGED		0x100

class CTabSwitcher
{
private:
	CTArray<_TSWITEM>	m_items;
	int					m_BtnSize;
	int					m_TextSize;
	BOOL				m_mouseTrackInstalled;
protected:
	HWND	m_hWnd;
	HFONT	m_hFont;
public:
	CTabSwitcher(void);
	virtual ~CTabSwitcher(void);

	static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);
protected:
	virtual void OnPaint(HDC hdc);
	virtual void OnCreate(void);
	virtual void OnMouseMove(int x, int y);
	virtual void OnLButtonDown(int x, int y);
private:
	void OnInsertItem(TSWITEM* item);
	void DrawItem(HDC hdc, _TSWITEM* item, LPRECT rc);
	void RecalcLayout(void);
	LRESULT OnNCCalcSize(WPARAM wParam, LPARAM lParam);
	void OnMouseLeave(void);
};

class CTabLabel
{
	HWND	m_hWnd;
	HFONT	m_hFont;
public:
	CTabLabel();
	~CTabLabel();

	static LRESULT CALLBACK WndProc(HWND hWnd, UINT uMessage, WPARAM wParam, LPARAM lParam);
protected:
	void OnPaint(HDC hdc);
	void OnCreate();
};

extern void RegisterTabSwitcherClass(HINSTANCE hInst);

inline COLORREF tswLightColor(COLORREF clr, long level)
{
	BYTE red   = GetRValue(clr);
	BYTE green = GetGValue(clr);
	BYTE blue  = GetBValue(clr);
	red = (BYTE) max(0, min(255, (int)((red - 128) + 128 + level)));
	green = (BYTE) max(0, min(255, (int)((green - 128) + 128 + level)));
	blue = (BYTE) max(0, min(255, (int)((blue - 128) + 128 + level)));
	return RGB(red, green, blue);
}
