#include "globals.h"
#include ".\pluginbtn.h"
#include "PropsDlg.h"
#include "TabSwitcher.h"
#include "TlbHotKeyCtl.h"
#include "PluginStream.h"

CPluginBtn::CPluginBtn(void)
{
	m_tipIcon	= NULL;
	m_hotKey	= 0;
}

CPluginBtn::~CPluginBtn(void)
{
	if(m_tipIcon)
	{
		DestroyIcon(m_tipIcon);
		m_tipIcon = NULL;
	}
}

BOOL CPluginBtn::OnCreate()
{
	CPropsDlg dlg;
	if(dlg.DoModal(m_container->GetToolbarWindow(), m_hResModule) == IDOK)
	{
		return TRUE;
	}
	return FALSE;
}

BOOL CPluginBtn::OnSetLCID(DWORD dwLCID, HMODULE hInstance)
{
    BOOL ret = CTlbButton::OnSetLCID(dwLCID, hInstance);
	RegisterTlbHotKeyClass(m_hResModule);
	RegisterTabSwitcherClass(m_hResModule);
	return ret;
}

BOOL CPluginBtn::LoadIcon(int size)
{
	if(!CTlbButton::LoadIcon(size))
	{
		m_hIcon = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, size, size, LR_DEFAULTCOLOR);
		m_loadedIconSize = size;
	}
	return m_hIcon ? TRUE : FALSE;
}

BOOL CPluginBtn::SetIconLocation(LPCWSTR szLocation)
{
	if(m_tipIcon) DestroyIcon(m_tipIcon);
	m_tipIcon = NULL;
	if(m_hIcon) DestroyIcon(m_hIcon);
	m_hIcon = NULL;
	if(m_pngIcon)
	{
		delete m_pngIcon;
		m_pngIcon = NULL;
	}
	return CTlbButton::SetIconLocation(szLocation);
}

HICON CPluginBtn::GetTipIcon(int tipID)
{
	if(!m_tipIcon)
	{
		m_tipIcon = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_ICON), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR | LR_LOADTRANSPARENT);
	}
	return m_tipIcon;
}

LPWSTR CPluginBtn::GetTipCaption(int tipID)
{
	LPWSTR name = (LPWSTR) CoTaskMemAlloc((lstrlen(m_Name) + 1) * sizeof(TCHAR));
	lstrcpy(name, m_Name);
	return name;
}

LPWSTR CPluginBtn::GetTipText(int tipID)
{
	//LPWSTR name = (LPWSTR) CoTaskMemAlloc((lstrlen(m_Description) + 1) * sizeof(TCHAR));
	//lstrcpy(name, m_Description);
	//return name;
	return NULL;
}

BOOL CPluginBtn::Load(IStream *data)
{ 
	CPluginStream stream;
	if(stream.Load(data))
	{
		if(stream.OpenNode(TEXT("settings")))
		{
			MAKE_STR(m_iconLocation, stream.GetString(TEXT("iconLocation")));
			m_hotKey = stream.GetWORD(TEXT("runhotkey"), 0);
			stream.CloseNode();

			if(m_hotKey)
			{
				m_container->SetHotKey(m_hotKey, HOTKEY_RUN);
			}
		}
	}

	return TRUE; 
}

BOOL CPluginBtn::Save(IStream *data)
{ 
	CPluginStream stream;
	stream.BeginNode(TEXT("settings"));
		stream.SaveString(TEXT("iconLocation"),	m_iconLocation);
		stream.SaveWORD(TEXT("runhotkey"), m_hotKey);
	stream.EndNode();

	stream.Save(data);
	return TRUE; 
}

UINT CPluginBtn::GetSupportedActions(void)
{
	return CTMS_PROPERTIES | CTMS_CHANGEICON;
}

UINT CPluginBtn::GetModeFlags(void)
{
	return BTN_FLAG_SUPPORTSAVEDICON | BTN_FLAG_SUPPORTPNGICONS;
}

BOOL CPluginBtn::OnProperties()
{
	CPropsDlg dlg;
	dlg.m_hotKey = m_hotKey;
	m_container->SetHotKey(0, HOTKEY_RUN);
	if(dlg.DoModal(m_container->GetToolbarWindow(), m_hResModule) == IDOK)
	{
		m_hotKey = dlg.m_hotKey;
		m_container->SetHotKey(m_hotKey, HOTKEY_RUN);
		m_container->UpdateTips();
		m_container->SaveButton();
		return TRUE;
	}
	m_container->SetHotKey(m_hotKey, HOTKEY_RUN);
	return FALSE;
}

BOOL CPluginBtn::OnRun()
{
	MessageBox(NULL, TEXT("Plugin Runs"), TEXT("plugin"), MB_OK | MB_ICONINFORMATION);
	return TRUE;
}

void CPluginBtn::OnHotkey(UINT hkID)
{
	switch(hkID)
	{
	case HOTKEY_RUN:
		OnRun();
		break;
	}
}

int CPluginBtn::GetHotkeysCount(void)
{
	return 1;
}

BOOL CPluginBtn::GetHotKeyData(int idx, HOTKEY_DATA* hkData)
{
	switch(idx)
	{
	case 0:
		hkData->hotKey = m_hotKey;
		hkData->hotKeyID = HOTKEY_RUN;
		hkData->id = HOTKEY_RUN;
		LoadString(m_hResModule, IDS_RUNHOTKEY, hkData->name, 260);
		break;
	}
	return TRUE;
}
