#include "globals.h"
#include ".\windowbtn.h"
#include "WndLstBtn.h"
#include <tlbpdklib.h>
#include <string>
#include "resource.h"

CWindowBtn::CWindowBtn(CWndLstBtn* parent, HWND hWnd, LPTSTR caption, HICON smallIcon, HICON largeIcon, LPTSTR processName)
{
	m_parent		= parent;
	m_hWnd			= hWnd;
	m_hSmallIcon	= smallIcon;
	m_hLargeIcon	= largeIcon;
	m_caption		= NULL;
	MAKE_STR(m_caption, caption);
	lstrcpy(m_processName, processName);
}

CWindowBtn::~CWindowBtn(void)
{
	FREE_CLEAR_STR(m_caption);
}

void CWindowBtn::GetChildUID(LPWSTR uid)
{
	_stprintf(uid, TEXT("wnd:%p"), m_hWnd);
}

void CWindowBtn::GetButtonText(LPWSTR text)
{
	StringCchCopy(text, MAX_PATH, m_caption);
}

void CWindowBtn::OnDraw(HDC hDC, LPRECT rcItem)
{
	int iconLeft = 0;
	int iconTop = 0;
	DWORD btnType = 0;
	if(m_container)
	{
		POINT pt;
		m_container->GetIconPoint(&pt);
		iconLeft = rcItem->left + pt.x;
		iconTop = rcItem->top + pt.y;
		btnType = m_container->GetButtonType();
	}

	if(!btnType || !(btnType & TS_ITEM_NOICON))
	{
		int szIcon = m_container->GetIconSize();
		HICON icon = NULL;
		if(szIcon < 32)
		{
			icon = m_hSmallIcon;
			if(!icon) icon = m_hLargeIcon;
		} else
		{
			icon = m_hLargeIcon;
			if(!icon) icon = m_hSmallIcon;
		}
		if(icon)
		{
			DrawIconEx((HDC) hDC, iconLeft, iconTop, icon, szIcon, szIcon, 0, 0, DI_NORMAL);
		}
	}
}

LPWSTR CWindowBtn::GetTipCaption(int tipID)
{
	LPWSTR name = (LPWSTR) CoTaskMemAlloc((lstrlen(m_caption) + 1) * sizeof(TCHAR));
	lstrcpy(name, m_caption);
	return name;
}

LPWSTR CWindowBtn::GetTipText(int tipID)
{
	TCHAR procNameLabel[MAX_PATH];
	LoadString(m_hResModule, IDS_TIPS_PROCESSNAME, procNameLabel, MAX_PATH);

	std::wstring str;
	str.append(procNameLabel);
	str.append(TEXT(":<t><b>"));
	str.append(m_processName);
	str.append(TEXT("</b>"));

	LPWSTR name = (LPWSTR) CoTaskMemAlloc((str.length() + 1) * sizeof(TCHAR));
	lstrcpy(name, str.c_str());
	return name;
}

HICON CWindowBtn::GetTipIcon(int tipID)
{
	return m_hLargeIcon;
}

UINT CWindowBtn::ReadChanges(void)
{
	if(m_parent->isDeleted(m_hWnd))
	{
		return 2;
	}
	WND_CHANGES changes;
	if(m_parent->getWindowChanges(m_hWnd, &changes))
	{
		FREE_CLEAR_STR(m_caption);
		m_caption = changes.m_caption;
		m_hLargeIcon = changes.m_large;
		m_hSmallIcon = changes.m_small;
		return READ_CHANGES_REDRAW | READ_CHANGES_RECALC | READ_CHANGES_RESORT;
	}
	return 0;
}

typedef VOID (WINAPI *SwitchToThisWindowProc)(HWND hWnd, BOOL fAltTab);

BOOL CWindowBtn::OnRun()
{
	HMODULE user32 = LoadLibrary(TEXT("user32.dll"));
	SwitchToThisWindowProc proc = (SwitchToThisWindowProc) GetProcAddress(user32, "SwitchToThisWindow");
	if(proc)
	{
		proc(m_hWnd, TRUE);
	}
	FreeLibrary(user32);
	return TRUE;
}

struct
{
	UINT	id;
	UINT	strID;
} g_ctxmenu[] = 
{
	{MID_WND_RESTORE,	IDS_WND_MENU_RESTORE},
	{MID_WND_MINIMIZE,	IDS_WND_MENU_MINIMIZE},
	{MID_WND_MAXIMIZE,	IDS_WND_MENU_MAXIMIZE},
	{0xFFFFFFFF,		0xFFFFFFFF},
	{MID_WND_CLOSE,		IDS_WND_MENU_CLOSE},
	{MID_WND_CLOSEAPP,	IDS_WND_MENU_CLOSEAPP},
	{0,		0}
};

int CWindowBtn::QueryContextMenu(HMENU hMenu, int index, int cmdFirst, int cmdLast)
{
	MENUITEMINFO mi;
	ZeroMemory(&mi, sizeof(mi));
	mi.cbSize = sizeof(MENUITEMINFO);

	int cnt = 0;
	for(int i=0; g_ctxmenu[i].id; i++)
	{
		if(g_ctxmenu[i].id == 0xFFFFFFFF)
		{
			InsertMenu( hMenu, 
				index++, 
				MF_SEPARATOR, 
				cmdFirst, 
				TEXT(""));
		} else
		{
			TCHAR menuItem[MAX_PATH];
			LoadString(m_hResModule, g_ctxmenu[i].strID, menuItem, MAX_PATH);
			mi.fMask = MIIM_TYPE | MIIM_ID;
			mi.fType = MFT_STRING;
			mi.dwTypeData = menuItem;
			mi.cch = lstrlen(mi.dwTypeData);
			mi.wID = cmdFirst + g_ctxmenu[i].id;
			InsertMenuItem(hMenu, index++, TRUE, &mi);
		}
		cnt++;
	}

	return cnt;
}

BOOL CWindowBtn::OnContextMenuCommand(int ID)
{
	UINT cmd = 0;
	switch(ID)
	{
	case MID_WND_RESTORE:
		cmd = SC_RESTORE;
		break;
	case MID_WND_MAXIMIZE:
		cmd = SC_MAXIMIZE;
		break;
	case MID_WND_MINIMIZE:
		cmd = SC_MINIMIZE;
		break;
	case MID_WND_MOVE:
		cmd = SC_MOVE;
		break;
	case MID_WND_SIZE:
		cmd = SC_SIZE;
		break;
	case MID_WND_CLOSE:
		cmd = SC_CLOSE;
		break;
	case MID_WND_CLOSEAPP:
		m_parent->CloseApp(m_processName);
		break;
	}
	if(cmd)
	{
		PostMessage(m_hWnd, WM_SYSCOMMAND, cmd, 0);
	}
	return TRUE;
}
