#include "globals.h"
#include ".\wndlstbtn.h"
#include "resource.h"
#include "WindowBtn.h"
#include <malloc.h>
#include <Psapi.h>

void GetProcessName(DWORD processID, LPTSTR szProcessName)
{
	lstrcpy(szProcessName, TEXT("unknown"));

	// Get a handle to the process.

	HANDLE hProcess = OpenProcess( PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, processID );

	// Get the process name.

	if (NULL != hProcess )
	{
		HMODULE hMod;
		DWORD cbNeeded;

		if (EnumProcessModules( hProcess, &hMod, sizeof(hMod), &cbNeeded))
		{
			GetModuleBaseName(hProcess, hMod, szProcessName, MAX_PATH);
		}
		else return;
	}
	else return;

	CloseHandle( hProcess );
}

HICON GetWindowIcon(HWND hWnd, BOOL isSmall)
{
	HICON ret = NULL;
	if(isSmall)
	{
		ret	= (HICON) SendMessage(hWnd, WM_GETICON, ICON_SMALL, NULL);
		if(!ret)
		{
			ret = (HICON) GetClassLongPtr(hWnd, GCLP_HICONSM);
		}
	} else
	{
		ret	= (HICON) SendMessage(hWnd, WM_GETICON, ICON_BIG,	NULL);
		if(!ret)
		{
			ret = (HICON) GetClassLongPtr(hWnd, GCLP_HICON);
		}
	}
	if(!ret)
	{
		HWND owner = GetWindow(hWnd, GW_OWNER);
		if(owner)
		{
			ret = GetWindowIcon(owner, isSmall);
		}
	}
	return ret;
}

void GetWindowImagePath(HWND hWnd, LPTSTR imagePath)
{
	TCHAR path[MAX_PATH];
	path[0] = 0;
	DWORD processID = 0;
	DWORD threadID = GetWindowThreadProcessId(hWnd, &processID);
	GetProcessName(processID, imagePath);
}

BOOL isAltTabWindow(HWND hWnd)
{
	DWORD style = GetWindowLongPtr(hWnd, GWL_STYLE);
	DWORD exStyle = GetWindowLongPtr(hWnd, GWL_EXSTYLE);

	if((exStyle & WS_EX_TOOLWINDOW) == WS_EX_TOOLWINDOW)
	{
		return FALSE;
	}
	if(style & WS_VISIBLE)
	{
		if(!(style & WS_DISABLED))
		{
			//if(style & WS_CAPTION)
			{
				RECT rcWindow;
				GetWindowRect(hWnd, &rcWindow);
				if(rcWindow.bottom - rcWindow.top > 0 && rcWindow.right - rcWindow.left > 0)
				{
					HWND hWndOner = GetWindow(hWnd, GW_OWNER);
					if(hWndOner)
					{
						HWND hwndLastActive = GetLastActivePopup(hWndOner);
						if(hwndLastActive != hWnd)
						{
							if(!(style & WS_POPUP))
							{
								return TRUE;
							} else
							{
								style = GetWindowLongPtr(hwndLastActive, GWL_STYLE);
								if(!(style & WS_VISIBLE) || (style & WS_DISABLED))
								{
									return TRUE;
								}
							}
						} else
						{
							if((exStyle & 0xF0000) || (exStyle & 0x1))
							{
								return TRUE;
							} else
							{
								exStyle = GetWindowLongPtr(hWndOner, GWL_EXSTYLE);
								if((exStyle & 0xF0000) || (exStyle & 0x1))
								{
									return TRUE;
								}
							}
						}
					} else
					{
						HWND hwndLastActive = GetLastActivePopup(hWnd);
						if(hwndLastActive != hWnd)
						{
							style = GetWindowLongPtr(hwndLastActive, GWL_STYLE);
							if(!(style & WS_POPUP) || (style & WS_DISABLED) || !(style & WS_VISIBLE))
							{
								return TRUE;
							}
						} else
						{
							return TRUE;
						}
					}
				}
			}
		}
	}
	return FALSE;
}

CWndLstBtn::CWndLstBtn(void)
{
	m_tipIcon = NULL;
	InitializeCriticalSection(&m_sync);
	m_hTimerThread = NULL;
	m_hStop = CreateEvent(NULL, FALSE, FALSE, NULL);
	m_changesHandle = NULL;
}

CWndLstBtn::~CWndLstBtn(void)
{
	StopTimer();
	if(m_tipIcon)
	{
		DestroyIcon(m_tipIcon);
		m_tipIcon = NULL;
	}
	ClearItems();
	CloseHandle(m_hStop);
	DeleteCriticalSection(&m_sync);
}


void CWndLstBtn::LockItems()
{
	EnterCriticalSection(&m_sync);
}

void CWndLstBtn::UnlockItems()
{
	LeaveCriticalSection(&m_sync);
}

void CWndLstBtn::ClearItems()
{
	LockItems();

	for(int i=0; i < m_windows.GetCount(); i++)
	{
		FREE_CLEAR_STR(m_windows[i].m_caption);
	}
	m_windows.Clear();

	UnlockItems();
}

BOOL CWndLstBtn::isMenu()
{
	return TRUE;
}

BOOL CWndLstBtn::Save(IStream *data)
{
	return TRUE;
}

BOOL CWndLstBtn::Load(IStream *data)
{
	StartTimer();
	return TRUE;
}

void CWndLstBtn::GetButtonText(LPWSTR text)
{
	StringCchCopy(text, MAX_PATH, m_Name);
/*	if(m_newCount)
	{
		TCHAR strCnt[20];
		wsprintf(strCnt, TEXT("%d"), m_newCount);
		StringCchCat(text, MAX_PATH, TEXT(" ("));
		StringCchCat(text, MAX_PATH, strCnt);
		StringCchCat(text, MAX_PATH, TEXT(")"));
	}*/
}

BOOL CWndLstBtn::LoadIcon(int size)
{
	if(!CTlbButton::LoadIcon(size))
	{
		m_hIcon = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_WNDLIST), IMAGE_ICON, size, size, LR_DEFAULTCOLOR);
		m_loadedIconSize = size;
	}
	return m_hIcon ? TRUE : FALSE;
}

BOOL CWndLstBtn::SetIconLocation(LPCWSTR szLocation)
{
	m_hIcon = NULL;
	if(m_pngIcon)
	{
		delete m_pngIcon;
		m_pngIcon = NULL;
	}
	return CTlbButton::SetIconLocation(szLocation);
}

BOOL CWndLstBtn::OnCreate()
{
	return TRUE;
}

UINT CWndLstBtn::GetSupportedActions()
{
	return CTlbButton::GetSupportedActions() | CTMS_CHANGEICON;
}

UINT CWndLstBtn::GetModeFlags()
{
	return CTlbButton::GetModeFlags() | BTN_FLAG_SUPPORTPNGICONS;
}

HICON CWndLstBtn::GetTipIcon(int tipID)
{
	if(!m_tipIcon)
	{
		m_tipIcon = (HICON) LoadImage(g_hInst, MAKEINTRESOURCE(IDI_WNDLIST), IMAGE_ICON, 32, 32, LR_DEFAULTCOLOR | LR_LOADTRANSPARENT);
	}
	return m_tipIcon;
}

DWORD WINAPI CWndLstBtn::TimerTrdProc(LPVOID lpParam)
{
	CoInitialize(NULL);
	CWndLstBtn* tr = (CWndLstBtn*) lpParam;
	tr->TimerThread();
	CoUninitialize();
	return 0;
}

BOOL CALLBACK CWndLstBtn::EnumWindowsProc(HWND hwnd, LPARAM lParam)
{
	if(isAltTabWindow(hwnd))
	{
		CWndLstBtn* pThis = (CWndLstBtn*) lParam;
		pThis->LockItems();
		BOOL found = FALSE;
		for(int i = 0; i < pThis->m_windows.GetCount(); i++)
		{
			if(pThis->m_windows[i].m_hWnd == hwnd)
			{
				found = TRUE;
				pThis->m_windows[i].isDeleted = FALSE;
				int szCaption = GetWindowTextLength(hwnd) + 1;
				LPTSTR caption = (LPTSTR) _alloca(szCaption * sizeof(TCHAR));
				GetWindowText(hwnd, caption, szCaption);
				if(lstrcmp(caption, pThis->m_windows[i].m_caption))
				{
					MAKE_STR(pThis->m_windows[i].m_caption, caption);
					pThis->m_windows[i].isChanged = TRUE;
				}
				HICON hSmall = GetWindowIcon(hwnd, TRUE);
				HICON hLarge = GetWindowIcon(hwnd, FALSE);
				if(hSmall != pThis->m_windows[i].m_hSmallIcon)
				{
					pThis->m_windows[i].m_hSmallIcon = hSmall;
					pThis->m_windows[i].isChanged = TRUE;
				}
				if(hLarge != pThis->m_windows[i].m_hLargeIcon)
				{
					pThis->m_windows[i].m_hLargeIcon = hLarge;
					pThis->m_windows[i].isChanged = TRUE;
				}
				break;
			}
		}
		if(!found)
		{
			WINDOW_ITEM wnd;
			int szCaption = GetWindowTextLength(hwnd) + 1;
			wnd.m_caption = new TCHAR[szCaption];
			GetWindowText(hwnd, wnd.m_caption, szCaption);
			wnd.m_hWnd			= hwnd;
			wnd.isDeleted		= FALSE;
			wnd.isNew			= TRUE;
			wnd.isChanged		= FALSE;
			wnd.m_hSmallIcon	= GetWindowIcon(hwnd, TRUE);
			wnd.m_hLargeIcon	= GetWindowIcon(hwnd, FALSE);
			wnd.imageName[0] = 0;

			GetWindowImagePath(hwnd, wnd.imageName);

			pThis->m_windows.Add(&wnd);
		}
		pThis->UnlockItems();
	}
	return TRUE;
}


void CWndLstBtn::TimerThread()
{
	while(true)
	{
		DWORD res = WaitForSingleObject(m_hStop, 250);
		if(res == WAIT_OBJECT_0)
		{
			break;
		}
		LockItems();
		for(int i=0; i < m_windows.GetCount(); i++)
		{
			m_windows[i].isDeleted	= TRUE;
			m_windows[i].isNew		= FALSE;
		}
		UnlockItems();

		EnumWindows(CWndLstBtn::EnumWindowsProc, (LPARAM) this);
		BOOL changed = FALSE;

		LockItems();
		for(int i=0; i < m_windows.GetCount(); i++)
		{
			if(m_windows[i].isNew)
			{
				changed = TRUE;
			}
			if(m_windows[i].isChanged)
			{
				changed = TRUE;
			}
			if(m_windows[i].isDeleted)
			{
				changed = TRUE;
				FREE_CLEAR_STR(m_windows[i].m_caption);
				m_windows.Delete(i);
				i--;
			}
		}
		UnlockItems();
		
		if(changed && m_changesHandle)
		{
			SetEvent(m_changesHandle);
		}
	}
}

void CWndLstBtn::StartTimer()
{
	DWORD id;	
	m_hTimerThread = CreateThread(NULL, 0, CWndLstBtn::TimerTrdProc, (LPVOID) this, 0, &id);
}

void CWndLstBtn::StopTimer()
{
	if(m_hTimerThread)
	{
		SetEvent(m_hStop);
		WaitForSingleObject(m_hTimerThread, INFINITE);
		CloseHandle(m_hTimerThread);
		m_hTimerThread = NULL;
	}
}

HANDLE CWndLstBtn::GetChangesHandle() 
{ 
	if(!m_changesHandle)
	{
		m_changesHandle = CreateEvent(NULL, TRUE, FALSE, NULL);
	}
	return m_changesHandle; 
}

void CWndLstBtn::PrepareChanges(HANDLE hChanges)
{
	ResetEvent(hChanges);
}

void CWndLstBtn::CloseChangesHandle(HANDLE hChanges)
{
	CloseHandle(hChanges);
	m_changesHandle = NULL;
}

BOOL CWndLstBtn::FirstChildData(CHILDS_DATA* data)
{
	LockItems();
	if(!m_windows.GetCount())
	{
		UnlockItems();
		return FALSE;
	}

	data->childData = (LPVOID) m_windows[0].m_hWnd;
	data->enumData = (LPVOID) 1;
	if(data->Name)
	{
		_stprintf(data->Name, TEXT("wnd:%p"), m_windows[0].m_hWnd);
	}
	UnlockItems();
	return TRUE;
}

BOOL CWndLstBtn::NextChildData(CHILDS_DATA* data)
{
	int idx = (int) data->enumData;
	LockItems();

	BOOL ret = FALSE;

	if(idx >=0 && idx < m_windows.GetCount())
	{
		data->childData = m_windows[idx].m_hWnd;
		data->enumData = (LPVOID) (idx + 1);
		if(data->Name)
		{
			_stprintf(data->Name, TEXT("wnd:%p"), m_windows[idx].m_hWnd);
		}
		ret = TRUE;
	}

	UnlockItems();
	return ret;
}

BOOL CWndLstBtn::FreeChildData(CHILDS_DATA* data)
{
	return TRUE;
}

BOOL CWndLstBtn::CreateChild(CREATE_CHILD_DATA* childData)
{
	BOOL ret = FALSE;
	LockItems();

	for(int i=0; i < m_windows.GetCount(); i++)
	{
		if(m_windows[i].m_hWnd == (HWND) childData->childData)
		{
			childData->child = new CWindowBtn(this, m_windows[i].m_hWnd, m_windows[i].m_caption, m_windows[i].m_hSmallIcon, m_windows[i].m_hLargeIcon, m_windows[i].imageName);
			ret = TRUE;
			break;
		}
	}

	UnlockItems();
	return ret;
}

BOOL CWndLstBtn::isDeleted(HWND hWnd)
{
	BOOL ret = TRUE;
	LockItems();

	for(int i=0; i < m_windows.GetCount(); i++)
	{
		if(m_windows[i].m_hWnd == hWnd)
		{
			ret = FALSE;
			break;
		}
	}

	UnlockItems();
	return ret;
}

BOOL CWndLstBtn::getWindowChanges(HWND hWnd, WND_CHANGES* changes)
{
	ZeroMemory(changes, sizeof(WND_CHANGES));
	BOOL ret = FALSE;
	LockItems();

	for(int i=0; i < m_windows.GetCount(); i++)
	{
		if(m_windows[i].m_hWnd == hWnd)
		{
			if(m_windows[i].isChanged)
			{
				m_windows[i].isChanged = FALSE;
				MAKE_STR(changes->m_caption, m_windows[i].m_caption);
				changes->m_small = m_windows[i].m_hSmallIcon;
				changes->m_large = m_windows[i].m_hLargeIcon;
				ret = TRUE;
			}
			break;
		}
	}

	UnlockItems();
	return ret;
}

UINT CWndLstBtn::GetMenuFlags()
{
	return MENU_F_NORENAME;
}


BOOL CWndLstBtn::SupportSortMenu(void)
{
	return TRUE;
}

void CWndLstBtn::CompareButtons(COMPARE_BUTTONS_DATA* cpBtns)
{
	CWindowBtn* btn1 = (CWindowBtn*) cpBtns->btn1;
	CWindowBtn* btn2 = (CWindowBtn*) cpBtns->btn2;
	cpBtns->result = 0;
	switch(cpBtns->fnc)
	{
	case AMID_PROCNAME:
		cpBtns->result = StrCmpI(btn1->m_processName, btn2->m_processName);
		if(!cpBtns->result)
		{
			cpBtns->result = StrCmpI(btn1->m_caption, btn2->m_caption);
		}
		break;
	case AMID_CAPTION:
		cpBtns->result = StrCmpI(btn1->m_caption, btn2->m_caption);
		break;
	}
}

BOOL CWndLstBtn::QueryArrangeMenu(UINT idx, ARRANGE_MENU_DATA* qamData)
{
	switch(idx)
	{
	case AMID_PROCNAME - 1:
		qamData->sortID = AMID_PROCNAME;
		LoadString(m_hResModule, IDS_SORT_PROCESSNAME, qamData->name, 100);
		return TRUE;
	case AMID_CAPTION - 1:
		qamData->sortID = AMID_CAPTION;
		LoadString(m_hResModule, IDS_SORT_CAPTION, qamData->name, 100);
		return TRUE;
	}
	return FALSE;
}

void CWndLstBtn::CloseApp(LPTSTR appname)
{
	LockItems();
	for(int i=0; i < m_windows.GetCount(); i++)
	{
		if(!lstrcmpi(m_windows[i].imageName, appname))
		{
			PostMessage(m_windows[i].m_hWnd, WM_SYSCOMMAND, SC_CLOSE, 0);
		}
	}
	UnlockItems();
}
